@@ -1,6 +1,6 @@
 /*
  * jack output driver. This file is part of Shairport Sync.
- * Copyright (c) 2019 Mike Brady <mikebrady@iercom.net>,
+ * Copyright (c) 2019 -- 2024 Mike Brady <4265913+mikebrady@users.noreply.github.com>,
  *                    Jörn Nettingsmeier <nettings@luchtbeweging.nl>
  *
  * All rights reserved.
@@ -40,48 +40,28 @@ typedef jack_default_audio_sample_t sample_t;
 #define jack_sample_size sizeof(sample_t)
 
 // Two-channel, 32bit audio:
-static const int bytes_per_frame = NPORTS * jack_sample_size;
+const int bytes_per_frame = NPORTS * jack_sample_size;
 
-static pthread_mutex_t buffer_mutex = PTHREAD_MUTEX_INITIALIZER;
-static pthread_mutex_t client_mutex = PTHREAD_MUTEX_INITIALIZER;
-
-int jack_init(int, char **);
-void jack_deinit(void);
-void jack_start(int, int);
-int play(void *, int);
-void jack_stop(void);
-int jack_delay(long *);
-void jack_flush(void);
-
-audio_output audio_jack = {.name = "jack",
-                           .help = NULL,
-                           .init = &jack_init,
-                           .deinit = &jack_deinit,
-                           .prepare = NULL,
-                           .start = &jack_start,
-                           .stop = NULL,
-                           .is_running = NULL,
-                           .flush = &jack_flush,
-                           .delay = &jack_delay,
-                           .play = &play,
-                           .volume = NULL,
-                           .parameters = NULL,
-                           .mute = NULL};
+pthread_mutex_t buffer_mutex = PTHREAD_MUTEX_INITIALIZER;
+pthread_mutex_t client_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 // This also affects deinterlacing.
 // So make it exactly the number of incoming audio channels!
-static jack_port_t *port[NPORTS];
-static const char *port_name[NPORTS] = {"out_L", "out_R"};
+jack_port_t *port[NPORTS];
+const char *port_name[NPORTS] = {"out_L", "out_R"};
 
-static jack_client_t *client;
-static jack_nframes_t sample_rate;
-static jack_nframes_t jack_latency;
 
-static jack_ringbuffer_t *jackbuf;
-static int flush_please = 0;
+int sps_sample_rate;
 
-static jack_latency_range_t latest_latency_range[NPORTS];
-static int64_t time_of_latest_transfer;
+jack_client_t *client;
+jack_nframes_t jack_sample_rate;
+jack_nframes_t jack_latency;
+
+jack_ringbuffer_t *jackbuf;
+int flush_please = 0;
+
+jack_latency_range_t latest_latency_range[NPORTS];
+int64_t time_of_latest_transfer;
 
 #ifdef CONFIG_SOXR
 typedef struct soxr_quality {
@@ -89,9 +69,9 @@ typedef struct soxr_quality {
   const char *name;
 } soxr_quality_t;
 
-static soxr_quality_t soxr_quality_table[] = {{SOXR_VHQ, "very high"}, {SOXR_HQ, "high"},
-                                              {SOXR_MQ, "medium"},     {SOXR_LQ, "low"},
-                                              {SOXR_QQ, "quick"},      {-1, NULL}};
+soxr_quality_t soxr_quality_table[] = {{SOXR_VHQ, "very high"}, {SOXR_HQ, "high"},
+                                       {SOXR_MQ, "medium"},     {SOXR_LQ, "low"},
+                                       {SOXR_QQ, "quick"},      {-1, NULL}};
 
 static int parse_soxr_quality_name(const char *name) {
   for (soxr_quality_t *s = soxr_quality_table; s->name != NULL; ++s) {
@@ -102,18 +82,13 @@ static int parse_soxr_quality_name(const char *name) {
   return -1;
 }
 
-static soxr_t soxr = NULL;
-static soxr_quality_spec_t quality_spec;
-static soxr_io_spec_t io_spec;
+soxr_t soxr = NULL;
+soxr_quality_spec_t quality_spec;
+soxr_io_spec_t io_spec;
 #endif
 
 static inline sample_t sample_conv(short sample) {
-  // It sounds correct, but I don't understand it.
-  // Zero int needs to be zero float. Check.
-  // Plus 32767 int is 1.0. Check.
-  // Minus 32767 int is -0.99997. And here my brain shuts down.
-  // In my head, it should be 1.0, and we should tolerate an overflow
-  // at minus 32768. But I'm sure there's a textbook explanation somewhere.
+  // signed 16-bit int to float
   return ((sample < 0) ? (-1.0 * sample / SHRT_MIN) : (1.0 * sample / SHRT_MAX));
 }
 
@@ -203,7 +178,7 @@ static void error(const char *desc) { warn("JACK error: \"%s\"", desc); }
 // This is the function JACK will call in case of a non-critical event in the library.
 static void info(const char *desc) { inform("JACK information: \"%s\"", desc); }
 
-int jack_init(__attribute__((unused)) int argc, __attribute__((unused)) char **argv) {
+static int jack_init(__attribute__((unused)) int argc, __attribute__((unused)) char **argv) {
   int i;
   int bufsz = -1;
   config.audio_backend_latency_offset = 0;
@@ -258,17 +233,17 @@ int jack_init(__attribute__((unused)) int argc, __attribute__((unused)) char **a
   if (!client) {
     die("Could not start JACK server. JackStatus is %x", status);
   }
-  sample_rate = jack_get_sample_rate(client);
+  jack_sample_rate = jack_get_sample_rate(client);
 #ifdef CONFIG_SOXR
   if (config.jack_soxr_resample_quality >= SOXR_QQ) {
     quality_spec = soxr_quality_spec(config.jack_soxr_resample_quality, 0);
     io_spec = soxr_io_spec(SOXR_INT16_I, SOXR_FLOAT32_I);
   } else
 #endif
-      if (sample_rate != 44100) {
+      if (jack_sample_rate != 44100) {
     die("The JACK server is running at the wrong sample rate (%d) for Shairport Sync."
         " Must be 44100 Hz.",
-        sample_rate);
+        jack_sample_rate);
   }
   jack_set_process_callback(client, &process, NULL);
   jack_set_graph_order_callback(client, &graph, NULL);
@@ -321,7 +296,8 @@ int jack_init(__attribute__((unused)) int argc, __attribute__((unused)) char **a
       }
       while (port_list[i++] != NULL) {
         inform(
-            "Additional matching port %s found. Check that the connections are what you intended.");
+            "Additional matching port %s found. Check that the connections are what you intended.",
+            port_list[i - 1]);
       }
       jack_free(port_list);
     }
@@ -331,7 +307,7 @@ int jack_init(__attribute__((unused)) int argc, __attribute__((unused)) char **a
   return 0;
 }
 
-void jack_deinit() {
+static void jack_deinit() {
   pthread_mutex_lock(&client_mutex);
   if (jack_deactivate(client))
     warn("Error deactivating jack client");
@@ -347,10 +323,11 @@ void jack_deinit() {
 #endif
 }
 
-void jack_start(int i_sample_rate, __attribute__((unused)) int i_sample_format) {
+static void jack_start(int i_sample_rate, __attribute__((unused)) int i_sample_format) {
   // Nothing to do, JACK client has already been set up at jack_init().
   // Also, we have no say over the sample rate or sample format of JACK,
   // We convert the 16bit samples to float, and die if the sample rate is != 44k1 without soxr.
+  sps_sample_rate = i_sample_rate;
 #ifdef CONFIG_SOXR
   if (config.jack_soxr_resample_quality >= SOXR_QQ) {
     // we might improve a bit with soxr_clear if the sample_rate doesn't change
@@ -358,7 +335,7 @@ void jack_start(int i_sample_rate, __attribute__((unused)) int i_sample_format)
       soxr_delete(soxr);
     }
     soxr_error_t e = NULL;
-    soxr = soxr_create(i_sample_rate, sample_rate, NPORTS, &e, &io_spec, &quality_spec, NULL);
+    soxr = soxr_create(sps_sample_rate, jack_sample_rate, NPORTS, &e, &io_spec, &quality_spec, NULL);
     if (!soxr) {
       die("Unable to create soxr resampler for JACK: %s", e);
     }
@@ -366,13 +343,13 @@ void jack_start(int i_sample_rate, __attribute__((unused)) int i_sample_format)
 #endif
 }
 
-void jack_flush() {
+static void jack_flush() {
   debug(2, "Only the consumer can safely flush a lock-free ringbuffer. Asking the"
            " process callback to do it...");
   flush_please = 1;
 }
 
-int jack_delay(long *the_delay) {
+static int jack_delay(long *the_delay) {
   // Semantics change: we now look at the last transfer into the lock-free
   // ringbuffer, not into the jack buffers directly (because locking those would
   // violate real-time constraints). On average, that should lead to  just a
@@ -388,17 +365,21 @@ int jack_delay(long *the_delay) {
   debug(2, "audio_occupancy_now is %d.", audio_occupancy_now);
   pthread_mutex_unlock(&buffer_mutex);
 
-  int64_t frames_processed_since_latest_latency_check = (delta * sample_rate) / 1000000000;
+  int64_t frames_processed_since_latest_latency_check = (delta * jack_sample_rate) / 1000000000;
   // debug(1,"delta: %" PRId64 " frames.",frames_processed_since_latest_latency_check);
   // jack_latency is set by the graph() callback, it's the average of the maximum
   // latencies of all our output ports. Adjust this constant baseline delay according
   // to the buffer fill level:
-  *the_delay = jack_latency + audio_occupancy_now - frames_processed_since_latest_latency_check;
-  // debug(1,"reporting a delay of %d frames",*the_delay);
+  int64_t the_delay_in_jack_frames = jack_latency + audio_occupancy_now - frames_processed_since_latest_latency_check;
+  int64_t the_delay_in_sps_frames = (the_delay_in_jack_frames * sps_sample_rate) / jack_sample_rate;
+  *the_delay = the_delay_in_sps_frames;
+  // debug(2, "reporting a delay of %ld frames at Shairport Sync's rate of %d FPS.",*the_delay, sps_sample_rate);
   return 0;
 }
 
-int play(void *buf, int samples) {
+static int play(void *buf, int samples, __attribute__((unused)) int sample_type,
+                __attribute__((unused)) uint32_t timestamp,
+                __attribute__((unused)) uint64_t playtime) {
   jack_ringbuffer_data_t v[2] = {0};
   size_t i, j, c;
   jack_nframes_t thisbuf;
@@ -444,3 +425,19 @@ int play(void *buf, int samples) {
   }
   return 0;
 }
+
+audio_output audio_jack = {.name = "jack",
+                           .help = NULL,
+                           .init = &jack_init,
+                           .deinit = &jack_deinit,
+                           .prepare = NULL,
+                           .start = &jack_start,
+                           .stop = NULL,
+                           .is_running = NULL,
+                           .flush = &jack_flush,
+                           .delay = &jack_delay,
+                           .stats = NULL,
+                           .play = &play,
+                           .volume = NULL,
+                           .parameters = NULL,
+                           .mute = NULL};
