@@ -206,7 +206,9 @@ int dacp_send_command(const char *command, char **body, ssize_t *bodysize) {
       pthread_cleanup_push(addrinfo_cleanup, (void *)&res);
       // only do this one at a time -- not sure it is necessary, but better safe than sorry
 
-      int mutex_reply = sps_pthread_mutex_timedlock(&dacp_conversation_lock, 2000000, command, 1);
+      // int mutex_reply = sps_pthread_mutex_timedlock(&dacp_conversation_lock, 2000000, command,
+      // 1);
+      int mutex_reply = debug_mutex_lock(&dacp_conversation_lock, 2000000, 1);
       // int mutex_reply = pthread_mutex_lock(&dacp_conversation_lock);
       if (mutex_reply == 0) {
         pthread_cleanup_push(mutex_lock_cleanup, (void *)&dacp_conversation_lock);
@@ -409,17 +411,17 @@ void set_dacp_server_information(rtsp_conn_info *conn) {
     debug(2, "set_dacp_server_information set IP to \"%s\" and DACP id to \"%s\".",
           dacp_server.ip_string, dacp_server.dacp_id);
 
-    // If the client is forked-daapd, then we always use revision number 1
-    // because otherwise the return read will hang in a "long poll" if there
-    // are no changes.
-    // This is different to other AirPlay clients
-    // which return immediately with a 403 code if there are no changes.
-    dacp_server.always_use_revision_number_1 = 0;
-    char *p = strstr(conn->UserAgent, "forked-daapd");
-    if ((p != 0) &&
-        (p == conn->UserAgent)) { // must exist and be at the start of the UserAgent string
-      dacp_server.always_use_revision_number_1 = 1;
-    }
+    /*
+
+    "long polling" is not implemented by Shairport Sync, whereby by sending the client the
+    last-received revision number, the link will hang until a change occurs.
+
+    Instead, at present, Shairport Sync always uses a revision number of 1.
+
+    */
+
+    // always use revision number 1
+    dacp_server.always_use_revision_number_1 = 1;
 
     metadata_hub_modify_prolog();
     int ch = metadata_store.dacp_server_active != dacp_server.scan_enable;
@@ -463,7 +465,7 @@ void dacp_monitor_port_update_callback(char *dacp_id, uint16_t port) {
         "dacp_monitor_port_update_callback with Remote ID \"%s\", target ID \"%s\" and port "
         "number %d.",
         dacp_id, dacp_server.dacp_id, port);
-  if ((dacp_id == NULL) || (dacp_server.dacp_id == NULL)) {
+  if ((dacp_id == NULL) || (dacp_server.dacp_id[0] == '\0')) {
     warn("dacp_id or dacp_server.dacp_id NULL detected");
   } else {
     if (strcmp(dacp_id, dacp_server.dacp_id) == 0) {
@@ -479,18 +481,13 @@ void dacp_monitor_port_update_callback(char *dacp_id, uint16_t port) {
       //    metadata_store.dacp_server_active = dacp_server.scan_enable;
       //    metadata_hub_modify_epilog(ch);
     } else {
-      debug(1, "dacp port monitor reporting on an out-of-use remote.");
+      debug(2, "dacp port monitor reporting on an out-of-use remote.");
     }
   }
   pthread_cond_signal(&dacp_server_information_cv);
   debug_mutex_unlock(&dacp_server_information_lock, 3);
 }
 
-void dacp_monitor_thread_code_cleanup(__attribute__((unused)) void *arg) {
-  // debug(1, "dacp_monitor_thread_code_cleanup called.");
-  pthread_mutex_unlock(&dacp_server_information_lock);
-}
-
 void *dacp_monitor_thread_code(__attribute__((unused)) void *na) {
   int scan_index = 0;
   int always_use_revision_number_1 = 0;
@@ -502,12 +499,8 @@ void *dacp_monitor_thread_code(__attribute__((unused)) void *na) {
   int idle_scan_count = 0;
   while (1) {
     int result = 0;
-    sps_pthread_mutex_timedlock(
-        &dacp_server_information_lock, 500000,
-        "dacp_monitor_thread_code couldn't get DACP server information lock in 0.5 second!.", 2);
     int32_t the_volume;
-
-    pthread_cleanup_push(dacp_monitor_thread_code_cleanup, NULL);
+    pthread_cleanup_debug_mutex_lock(&dacp_server_information_lock, 500000, 2);
     if (dacp_server.scan_enable == 0) {
       metadata_hub_modify_prolog();
       int ch = (metadata_store.dacp_server_active != 0) ||
@@ -527,7 +520,7 @@ void *dacp_monitor_thread_code(__attribute__((unused)) void *na) {
           (uint64_t)(1000000000 * config.missing_port_dacp_scan_interval_seconds);
 
 #ifdef COMPILE_FOR_LINUX_AND_FREEBSD_AND_CYGWIN_AND_OPENBSD
-      uint64_t time_of_wakeup_ns = get_absolute_time_in_ns() + time_to_wait_for_wakeup_ns;
+      uint64_t time_of_wakeup_ns = get_realtime_in_ns() + time_to_wait_for_wakeup_ns;
       uint64_t sec = time_of_wakeup_ns / 1000000000;
       uint64_t nsec = time_of_wakeup_ns % 1000000000;
 #endif
@@ -568,7 +561,7 @@ void *dacp_monitor_thread_code(__attribute__((unused)) void *na) {
     pthread_cleanup_pop(1);
 
     if (result == 490) { // 490 means no port was specified
-      if (((char *)dacp_server.dacp_id != NULL) && (strlen(dacp_server.dacp_id) != 0)) {
+      if (strlen(dacp_server.dacp_id) != 0) {
         // debug(1,"mdns_dacp_monitor_set_id");
         mdns_dacp_monitor_set_id(dacp_server.dacp_id);
       }
@@ -658,7 +651,7 @@ void *dacp_monitor_thread_code(__attribute__((unused)) void *na) {
         char *response = NULL;
         int32_t item_size;
         char command[1024] = "";
-        if (always_use_revision_number_1 != 0) // for forked-daapd
+        if (always_use_revision_number_1 != 0) // see the "long polling" note above
           revision_number = 1;
         snprintf(command, sizeof(command) - 1, "playstatusupdate?revision-number=%d",
                  revision_number);
@@ -808,8 +801,9 @@ void *dacp_monitor_thread_code(__attribute__((unused)) void *na) {
                 case 'canp': // nowplaying 4 ids: dbid, plid, playlistItem, itemid (from mellowware
                              // see reference above)
                   debug(2, "DACP Composite ID seen");
-                  if (memcmp(metadata_store.item_composite_id, sp - item_size,
-                             sizeof(metadata_store.item_composite_id)) != 0) {
+                  if ((metadata_store.item_composite_id_is_valid == 0) ||
+                      (memcmp(metadata_store.item_composite_id, sp - item_size,
+                              sizeof(metadata_store.item_composite_id)) != 0)) {
                     memcpy(metadata_store.item_composite_id, sp - item_size,
                            sizeof(metadata_store.item_composite_id));
                     char st[33];
@@ -822,6 +816,7 @@ void *dacp_monitor_thread_code(__attribute__((unused)) void *na) {
                     *pt = 0;
                     debug(2, "Item composite ID changed to 0x%s.", st);
                     metadata_store.item_composite_id_changed = 1;
+                    metadata_store.item_composite_id_is_valid = 1;
                   }
                   break;
                 case 'astm':
@@ -831,6 +826,7 @@ void *dacp_monitor_thread_code(__attribute__((unused)) void *na) {
                   if (ui != metadata_store.songtime_in_milliseconds) {
                     metadata_store.songtime_in_milliseconds = ui;
                     metadata_store.songtime_in_milliseconds_changed = 1;
+                    metadata_store.songtime_in_milliseconds_is_valid = 1;
                     debug(2, "DACP Song Time set to: \"%u\"",
                           metadata_store.songtime_in_milliseconds);
                   }
@@ -936,18 +932,9 @@ void *dacp_monitor_thread_code(__attribute__((unused)) void *na) {
 void dacp_monitor_start() {
   int rc;
 
-#ifdef COMPILE_FOR_LINUX_AND_FREEBSD_AND_CYGWIN_AND_OPENBSD
-  pthread_condattr_t attr;
-  pthread_condattr_init(&attr);
-  pthread_condattr_setclock(&attr, CLOCK_MONOTONIC); // can't do this in OS X, and don't need it.
-  rc = pthread_cond_init(&dacp_server_information_cv, &attr);
+  rc = pthread_cond_init(&dacp_server_information_cv, NULL);
   if (rc)
     debug(1, "Error initialising the DACP Server Information Condition Variable");
-  pthread_condattr_destroy(&attr);
-#endif
-#ifdef COMPILE_FOR_OSX
-  rc = pthread_cond_init(&dacp_server_information_cv, NULL);
-#endif
 
   pthread_mutexattr_t mta;
 
@@ -1233,8 +1220,8 @@ int dacp_get_volume(int32_t *the_actual_volume) {
     http_response = dacp_get_speaker_list((dacp_spkr_stuff *)&speaker_info, 50, &speaker_count);
     if (http_response == 200) {
       // get our machine number
-      uint16_t *hn = (uint16_t *)config.hw_addr;
-      uint32_t *ln = (uint32_t *)(config.hw_addr + 2);
+      uint16_t *hn = (uint16_t *)config.ap1_prefix;
+      uint32_t *ln = (uint32_t *)(config.ap1_prefix + 2);
       uint64_t t1 = ntohs(*hn);
       uint64_t t2 = ntohl(*ln);
       int64_t machine_number = (t1 << 32) + t2; // this form is useful
@@ -1287,8 +1274,8 @@ int dacp_set_volume(int32_t vo) {
       http_response = dacp_get_speaker_list((dacp_spkr_stuff *)&speaker_info, 50, &speaker_count);
       if (http_response == 200) {
         // get our machine number
-        uint16_t *hn = (uint16_t *)config.hw_addr;
-        uint32_t *ln = (uint32_t *)(config.hw_addr + 2);
+        uint16_t *hn = (uint16_t *)config.ap1_prefix;
+        uint32_t *ln = (uint32_t *)(config.ap1_prefix + 2);
         uint64_t t1 = ntohs(*hn);
         uint64_t t2 = ntohl(*ln);
         int64_t machine_number = (t1 << 32) + t2; // this form is useful
@@ -1358,12 +1345,14 @@ int dacp_set_volume(int32_t vo) {
                     machine_number,
                     highest_other_volume); // set the overall volume to the highest one
               }
-              int32_t desired_relative_volume =
-                  (vo * 100 + (highest_other_volume / 2)) / highest_other_volume;
-              debug(2, "Set our speaker volume relative to the highest volume.");
-              http_response = dacp_set_speaker_volume(
-                  machine_number,
-                  desired_relative_volume); // set the overall volume to the highest one
+              if (highest_other_volume != 0) {
+                int32_t desired_relative_volume =
+                    (vo * 100 + (highest_other_volume / 2)) / highest_other_volume;
+                debug(2, "Set our speaker volume relative to the highest volume.");
+                http_response = dacp_set_speaker_volume(
+                    machine_number,
+                    desired_relative_volume); // set the overall volume to the highest one
+              }
             }
           }
         }
