@@ -33,13 +33,13 @@ typedef struct {
 
 /*
  #define MDNS_RECORD_WITH_METADATA \
-  "tp=UDP", "sm=false", "ek=1", "et=0,1", "cn=0,1", "ch=2", METADATA_EXPRESSION, "ss=16",          \
+  "tp=UDP", "sm=false", "ek=1", "et=0,1", "cn=0,1", "ch=2", METADATA_EXPRESSION, "ss=16",                    \
       "sr=44100", "vn=3", "txtvers=1", config.password ? "pw=true" : "pw=false"
 */
 #define MDNS_RECORD_WITH_METADATA                                                                  \
   "sf=0x4", "fv=76400.10", "am=ShairportSync", "vs=105.1", "tp=TCP,UDP", "vn=65537",               \
-      METADATA_EXPRESSION, "ss=16", "sr=44100", "da=true", "sv=false", "et=0,1", "ek=1", "cn=0,1", \
-      "ch=2", "txtvers=1", config.password ? "pw=true" : "pw=false"
+      METADATA_EXPRESSION, "ss=16", "sr=44100", "da=true", "sv=false", "et=0,1", "ek=1", \
+      "cn=0,1", "ch=2", "txtvers=1", config.password ? "pw=true" : "pw=false"
 
 #endif
 
@@ -52,7 +52,7 @@ typedef struct {
 
 #define MDNS_RECORD_WITHOUT_METADATA                                                               \
   "sf=0x4", "fv=76400.10", "am=ShairportSync", "vs=105.1", "tp=TCP,UDP", "vn=65537", "ss=16",      \
-      "sr=44100", "da=true", "sv=false", "et=0,1", "ek=1", "cn=0,1", "ch=2", "txtvers=1",          \
+      "sr=44100", "da=true", "sv=false", "et=0,1", "ek=1", "cn=0,1", "ch=2", "txtvers=1",                    \
       config.password ? "pw=true" : "pw=false"
 
 #endif // _MDNS_H
