@@ -43,12 +43,24 @@ typedef struct metadata_bundle {
                    // server
   int client_ip_changed;
 
+  char *client_name; // the name of the client device, if available
+  int client_name_changed;
+
   char *server_ip; // IP number used by Shairport Sync
   int server_ip_changed;
 
+  char *stream_type; // Realtime or Buffered
+  int stream_type_changed;
+
   char *progress_string; // progress string, emitted by the source from time to time
   int progress_string_changed;
 
+  char *frame_position_string; // frame position string emitted by SPS on request
+  int frame_position_string_changed;
+
+  char *first_frame_position_string; // first frame position string emitted by SPS on request
+  int first_frame_position_string_changed;
+
   int player_thread_active; // true if a play thread is running
   int dacp_server_active;   // true if there's a reachable DACP server (assumed to be the Airplay
                             // client) ; false otherwise
@@ -70,11 +82,16 @@ typedef struct metadata_bundle {
 
   uint64_t item_id; // seems to be a track ID -- see itemid in DACP.c
   int item_id_changed;
-  int item_id_received; // important for deciding if the track information should be ignored.
+  int item_id_is_valid;
 
   unsigned char
       item_composite_id[16]; // seems to be nowplaying 4 ids: dbid, plid, playlistItem, itemid
   int item_composite_id_changed;
+  int item_composite_id_is_valid;
+
+  int song_data_kind;
+  int song_data_kind_changed;
+  int song_data_kind_is_valid;
 
   char *track_name;
   int track_name_changed;
@@ -120,6 +137,7 @@ typedef struct metadata_bundle {
 
   uint32_t songtime_in_milliseconds;
   int songtime_in_milliseconds_changed;
+  int songtime_in_milliseconds_is_valid;
 
   // end
 
