@@ -174,7 +174,7 @@ void mpris_metadata_watcher(struct metadata_bundle *argc, __attribute__((unused)
   }
 
   // Add in the Track ID based on the 'mper' metadata if it is non-zero
-  if (argc->item_id != 0) {
+  if (argc->item_id_is_valid != 0) {
     char trackidstring[128];
     snprintf(trackidstring, sizeof(trackidstring), "/org/gnome/ShairportSync/%" PRIX64 "",
              argc->item_id);
@@ -212,7 +212,7 @@ void mpris_metadata_watcher(struct metadata_bundle *argc, __attribute__((unused)
     g_variant_builder_add(dict_builder, "{sv}", "xesam:genre", genre);
   }
 
-  if (argc->songtime_in_milliseconds) {
+  if (argc->songtime_in_milliseconds_is_valid) {
     uint64_t track_length_in_microseconds = argc->songtime_in_milliseconds;
     track_length_in_microseconds *= 1000; // to microseconds in 64-bit precision
                                           // Make up the track name and album name
@@ -229,7 +229,8 @@ void mpris_metadata_watcher(struct metadata_bundle *argc, __attribute__((unused)
 static gboolean on_handle_quit(MediaPlayer2 *skeleton, GDBusMethodInvocation *invocation,
                                __attribute__((unused)) gpointer user_data) {
   debug(1, "quit requested (MPRIS interface).");
-  pthread_cancel(main_thread_id);
+  type_of_exit_cleanup = TOE_dbus; // request an exit cleanup that is compatible with dbus
+  exit(EXIT_SUCCESS);
   media_player2_complete_quit(skeleton, invocation);
   return TRUE;
 }
